'use strict';
const Plugin = require('perf.plugin');
const Soa = require('@ele/node-soa_v2');
const util = require('util');
const get = require('lodash.get');

/**
 * 
 *   {
 *     "baas": {
 *        "appId": "5938fdda6bfb735e1f31ef1d",
 *        "userId": "22311",
 *        "soa": {
 *          "appid": "fe.baas_api",
 *          "clusterName": "stable"
 *        },
 *        "tableName": "foody"
 *     },
 *     "filterTypes": ["weex"],
 *     "filterApps": ["onepiece"],
 *     "rule": {
 *        "key": 'query.d.success',
 *        "value": "false"
 *     }
 *   }
 */
module.exports = class baasOutputPlugin extends Plugin {
  constructor(config = {}) {
    super();
    this.baas = config.baas || {};
    this.rule = config.rule || {};
    this.filterTypes = config.filterTypes || ['foody'];
    this.filterApps = config.filterApps || ['*'];
  }
  
  connect() {
    let CONF = this.baas.soa;
    let appid = 'fe.baas_api';
    let clusterName = 'stable';

    if (CONF) {
      if (CONF.appid) appid = CONF.appid;
      if (CONF.clusterName) clusterName = CONF.clusterName;
    }
    console.log(appid, clusterName);
    return Soa({ appid, clusterName });
  }
  
  insert(client, msg) {
    let { appId, userId, tableName } = this.baas;
    
    if (appId && userId && tableName) {
      client.call('createRecord', {
        params: {
          appId,
          userId,
          tableName,
          body: {msg}
        }
      });
    }
  }

  insertMany() {}

  filter(data) {
    let type = util.isString(data.query.type) ? data.query.type : '';
    let appName = data.query.appName;
    let { key, value } = this.rule;
    key = get(data, key);
    let validType = this.filterTypes.includes(type);
    let validApp = this.filterApps.includes(appName) || this.filterApps.includes('*');
    let validRule = key && value && (key === value) ? true : false;

    if (validType && validApp && validRule) {
      this.connect()
      .then(client => {
        console.log(client);
        this.insert(client, data);
      })
      .catch(err => {
        console.error(err);
      });
    }

    return data;
  }

  handle(_, data) {
    let result = this.filter(data);
    
    return result ? super.handle(result) : null;
  }
};
